MAX_FILE_ID_COUNT = 500
MAX_FILE_NAME_COUNT = 500
MAX_SYMBOLIC_PATH_COUNT = 500
MAX_FILE_NAME_LENGTH = 64
SYMBOLIC_DIRECTORY_PATH_MAX_LENGTH = 128
TAGS_COUNT_MAX = 20
TAG_KEY_LENGTH_MAX = 32
TAG_VALUE_LENGTH_MAX = 32
MAX_FILES_COUNT = 10000
FILE_DESCRIPTION_MAX_LENGTH = 256
EXPIRE_TIME_MIN = 60
EXPIRE_TIME_MAX = 60 * 60 * 24 * 30
EXPIRE_TIME_DEFAULT = 60 * 30
EXPIRE_TIME_DEFAULT_SEARCH = 60 * 60 * 24 * 7
